@@ -2,8 +2,8 @@
 //  RQuantLib function BermudanSwaption
 //
 //  Copyright (C) 2005 - 2007  Dominick Samperi
-//  Copyright (C) 2007 - 2019  Dirk Eddelbuettel
-//  Copyright (C) 2016         Terry Leitch
+//  Copyright (C) 2007 - 2023  Dirk Eddelbuettel
+//  Copyright (C) 2016 - 2023  Terry Leitch
 //
 //  This file is part of RQuantLib.
 //
@@ -24,26 +24,28 @@
 
 // Calibrates underlying swaptions to the input volatility matrix.
 void calibrateModel(const QuantLib::ext::shared_ptr<QuantLib::ShortRateModel>& model,
-                    const std::vector<QuantLib::ext::shared_ptr<QuantLib::BlackCalibrationHelper> > &helpers,
+                    const std::vector<QuantLib::ext::shared_ptr<QuantLib::BlackCalibrationHelper> > &swaptions,
                     QuantLib::Real lambda,
                     Rcpp::NumericVector &swaptionMat,
                     Rcpp::NumericVector &swapLengths,
                     Rcpp::NumericMatrix &swaptionVols) {
 
+    std::vector<QuantLib::ext::shared_ptr<QuantLib::CalibrationHelper> >
+        helpers(swaptions.begin(), swaptions.end());
     QuantLib::Size numRows = swaptionVols.nrow();
     QuantLib::Size numCols = swaptionVols.ncol();
     QuantLib::LevenbergMarquardt om;
 
-    model->calibrate(helpers, om,QuantLib:: EndCriteria(400,100,1.0e-8, 1.0e-8, 1.0e-8));
+    model->calibrate(helpers, om, QuantLib:: EndCriteria(400,100,1.0e-8, 1.0e-8, 1.0e-8));
 
     // Output the implied Black volatilities
     for (QuantLib::Size i=0; i<numRows; i++) {
-	QuantLib::Real npv = helpers[i]->modelValue();
-	QuantLib::Volatility implied = helpers[i]->impliedVolatility(npv, 1e-4,
+	QuantLib::Real npv = swaptions[i]->modelValue();
+	QuantLib::Volatility implied = swaptions[i]->impliedVolatility(npv, 1e-4,
                                                            1000, 0.05, 1.50);
 	QuantLib::Volatility diff = implied - swaptionVols(i, numCols-i-1);
 
-	Rprintf((char*) "%dx%d: model %lf, market %lf, diff %lf\n",
+	Rprintf((char*) "%.0fx%.0f: model %lf, market %lf, diff %lf\n",
             swaptionMat[i], swapLengths[numCols-i-1], implied,
             swaptionVols(i, numCols-i-1), diff);
     }
@@ -51,10 +53,10 @@ void calibrateModel(const QuantLib::ext::shared_ptr<QuantLib::ShortRateModel>& m
 
 // [[Rcpp::export]]
 Rcpp::List bermudanFromYieldEngine(Rcpp::List rparam,
-                                  Rcpp::NumericVector yield,
-                                  Rcpp::NumericVector swaptionMat,
-                                  Rcpp::NumericVector swapLengths,
-                                  Rcpp::NumericMatrix swaptionVols) {
+                                   Rcpp::NumericVector yield,
+                                   Rcpp::NumericVector swaptionMat,
+                                   Rcpp::NumericVector swapLengths,
+                                   Rcpp::NumericMatrix swaptionVols) {
 
     QuantLib::Size i;
     //int *swaptionMat=0, *swapLengths=0;
@@ -358,12 +360,12 @@ Rcpp::List bermudanWithRebuiltCurveEngine(Rcpp::List rparam,
     }
 
 
-    boost::shared_ptr<QuantLib::VanillaSwap>
+    QuantLib::ext::shared_ptr<QuantLib::VanillaSwap>
         swap(new QuantLib::VanillaSwap(type, notional,
                                        fixedSchedule, dummyFixedRate, fixedLegDayCounter,
                                        floatSchedule, indexSixMonths, 0.0,
                                        indexSixMonths->dayCounter()));
-    swap->setPricingEngine(boost::shared_ptr<QuantLib::PricingEngine>(new QuantLib::DiscountingSwapEngine(rhTermStructure)));
+    swap->setPricingEngine(QuantLib::ext::shared_ptr<QuantLib::PricingEngine>(new QuantLib::DiscountingSwapEngine(rhTermStructure)));
 
     // Find the ATM or break-even rate
     QuantLib::Rate fixedATMRate = swap->fairRate();
@@ -375,12 +377,12 @@ Rcpp::List bermudanWithRebuiltCurveEngine(Rcpp::List rparam,
         fixedRate = strike;
 
     // The swap underlying the Bermudan swaption.
-    boost::shared_ptr<QuantLib::VanillaSwap>
+    QuantLib::ext::shared_ptr<QuantLib::VanillaSwap>
         mySwap(new QuantLib::VanillaSwap(type, notional,
                                          fixedSchedule, fixedRate, fixedLegDayCounter,
                                          floatSchedule, indexSixMonths, 0.0,
                                          indexSixMonths->dayCounter()));
-    swap->setPricingEngine(boost::shared_ptr<QuantLib::PricingEngine>(new QuantLib::DiscountingSwapEngine(rhTermStructure)));
+    swap->setPricingEngine(QuantLib::ext::shared_ptr<QuantLib::PricingEngine>(new QuantLib::DiscountingSwapEngine(rhTermStructure)));
 
     // Build swaptions that will be used to calibrate model to
     // the volatility matrix.
@@ -389,15 +391,15 @@ Rcpp::List bermudanWithRebuiltCurveEngine(Rcpp::List rparam,
         swaptionMaturities.push_back(QuantLib::Period(swaptionMat[i], QuantLib::Years));
 
     // Swaptions used for calibration
-    std::vector<boost::shared_ptr<QuantLib::BlackCalibrationHelper> > swaptions;
+    std::vector<QuantLib::ext::shared_ptr<QuantLib::BlackCalibrationHelper> > swaptions;
 
     // List of times that have to be included in the timegrid
     std::list<QuantLib::Time> times;
     for (i=0; i<(QuantLib::Size)numRows; i++) {
 
-        //boost::shared_ptr<QuantLib::Quote> vol(new QuantLib::SimpleQuote(swaptionVols[i][numCols-i-1]));
-        boost::shared_ptr<QuantLib::Quote> vol(new QuantLib::SimpleQuote(swaptionVols(i, numCols-i-1)));
-        swaptions.push_back(boost::shared_ptr<QuantLib::BlackCalibrationHelper>(new QuantLib::SwaptionHelper(swaptionMaturities[i],
+        //QuantLib::ext::shared_ptr<QuantLib::Quote> vol(new QuantLib::SimpleQuote(swaptionVols[i][numCols-i-1]));
+        QuantLib::ext::shared_ptr<QuantLib::Quote> vol(new QuantLib::SimpleQuote(swaptionVols(i, numCols-i-1)));
+        swaptions.push_back(QuantLib::ext::shared_ptr<QuantLib::BlackCalibrationHelper>(new QuantLib::SwaptionHelper(swaptionMaturities[i],
                                                                                                              QuantLib::Period(swapLengths[numCols-i-1], QuantLib::Years),
                                                                                                              QuantLib::Handle<QuantLib::Quote>(vol),
                                                                                                              indexSixMonths,
@@ -414,23 +416,23 @@ Rcpp::List bermudanWithRebuiltCurveEngine(Rcpp::List rparam,
 
     // Get Bermudan swaption exercise dates.
     std::vector<QuantLib::Date> bermudanDates;
-    const std::vector<boost::shared_ptr<QuantLib::CashFlow> >& leg = swap->fixedLeg();
-        for (i=0; i<leg.size(); i++) {
-        boost::shared_ptr<QuantLib::Coupon> coupon = boost::dynamic_pointer_cast<QuantLib::Coupon>(leg[i]);
+    const std::vector<QuantLib::ext::shared_ptr<QuantLib::CashFlow> >& leg = swap->fixedLeg();
+    for (i=0; i<leg.size(); i++) {
+        QuantLib::ext::shared_ptr<QuantLib::Coupon> coupon = QuantLib::ext::dynamic_pointer_cast<QuantLib::Coupon>(leg[i]);
         bermudanDates.push_back(coupon->accrualStartDate());
     }
 
-    boost::shared_ptr<QuantLib::Exercise> bermudaExercise(new QuantLib::BermudanExercise(bermudanDates));
+    QuantLib::ext::shared_ptr<QuantLib::Exercise> bermudaExercise(new QuantLib::BermudanExercise(bermudanDates));
 
     // Price based on method selected.
     if (method.compare("G2Analytic") == 0) {
 
-        boost::shared_ptr<QuantLib::G2> modelG2(new QuantLib::G2(rhTermStructure));
+        QuantLib::ext::shared_ptr<QuantLib::G2> modelG2(new QuantLib::G2(rhTermStructure));
         Rprintf((char*)"G2/Jamshidian (analytic) calibration\n");
         for(i = 0; i < swaptions.size(); i++)
-            swaptions[i]->setPricingEngine(boost::shared_ptr<QuantLib::PricingEngine>(new QuantLib::G2SwaptionEngine(modelG2, 6.0, 16)));
+          swaptions[i]->setPricingEngine(QuantLib::ext::shared_ptr<QuantLib::PricingEngine>(new QuantLib::G2SwaptionEngine(modelG2, 6.0, 16)));
         calibrateModel(modelG2, swaptions, 0.05, swaptionMat, swapLengths, swaptionVols);
-        boost::shared_ptr<QuantLib::PricingEngine> engine(new QuantLib::TreeSwaptionEngine(modelG2, 50));
+        QuantLib::ext::shared_ptr<QuantLib::PricingEngine> engine(new QuantLib::TreeSwaptionEngine(modelG2, 50));
         QuantLib::Swaption bermudanSwaption(mySwap, bermudaExercise);
         bermudanSwaption.setPricingEngine(engine);
         return Rcpp::List::create(Rcpp::Named("a")         = modelG2->params()[0],
@@ -444,12 +446,12 @@ Rcpp::List bermudanWithRebuiltCurveEngine(Rcpp::List rparam,
 
     } else if (method.compare("HWAnalytic") == 0) {
 
-        boost::shared_ptr<QuantLib::HullWhite> modelHW(new QuantLib::HullWhite(rhTermStructure));
+        QuantLib::ext::shared_ptr<QuantLib::HullWhite> modelHW(new QuantLib::HullWhite(rhTermStructure));
         Rprintf((char*)"Hull-White (analytic) calibration\n");
         for (i=0; i<swaptions.size(); i++)
-            swaptions[i]->setPricingEngine(boost::shared_ptr<QuantLib::PricingEngine>(new QuantLib::JamshidianSwaptionEngine(modelHW)));
+            swaptions[i]->setPricingEngine(QuantLib::ext::shared_ptr<QuantLib::PricingEngine>(new QuantLib::JamshidianSwaptionEngine(modelHW)));
         calibrateModel(modelHW, swaptions, 0.05, swaptionMat, swapLengths, swaptionVols);
-        boost::shared_ptr<QuantLib::PricingEngine> engine(new QuantLib::TreeSwaptionEngine(modelHW, 50));
+        QuantLib::ext::shared_ptr<QuantLib::PricingEngine> engine(new QuantLib::TreeSwaptionEngine(modelHW, 50));
         QuantLib::Swaption bermudanSwaption(mySwap, bermudaExercise);
         bermudanSwaption.setPricingEngine(engine);
         return Rcpp::List::create(Rcpp::Named("a") = modelHW->params()[0],
@@ -459,13 +461,13 @@ Rcpp::List bermudanWithRebuiltCurveEngine(Rcpp::List rparam,
         //Rcpp::Named("params") = params);
 
     } else if (method.compare("HWTree") == 0) {
-        boost::shared_ptr<QuantLib::HullWhite> modelHW2(new QuantLib::HullWhite(rhTermStructure));
+        QuantLib::ext::shared_ptr<QuantLib::HullWhite> modelHW2(new QuantLib::HullWhite(rhTermStructure));
         Rprintf((char*)"Hull-White (tree) calibration\n");
         for (i=0; i<swaptions.size(); i++)
-            swaptions[i]->setPricingEngine(boost::shared_ptr<QuantLib::PricingEngine>(new QuantLib::TreeSwaptionEngine(modelHW2,grid)));
+            swaptions[i]->setPricingEngine(QuantLib::ext::shared_ptr<QuantLib::PricingEngine>(new QuantLib::TreeSwaptionEngine(modelHW2,grid)));
 
         calibrateModel(modelHW2, swaptions, 0.05, swaptionMat, swapLengths, swaptionVols);
-        boost::shared_ptr<QuantLib::PricingEngine> engine(new QuantLib::TreeSwaptionEngine(modelHW2, 50));
+        QuantLib::ext::shared_ptr<QuantLib::PricingEngine> engine(new QuantLib::TreeSwaptionEngine(modelHW2, 50));
         QuantLib::Swaption bermudanSwaption(mySwap, bermudaExercise);
         bermudanSwaption.setPricingEngine(engine);
         return Rcpp::List::create(Rcpp::Named("a") = modelHW2->params()[0],
@@ -475,13 +477,13 @@ Rcpp::List bermudanWithRebuiltCurveEngine(Rcpp::List rparam,
         //Rcpp::Named("params") = params);
 
     } else if (method.compare("BKTree") == 0) {
-        boost::shared_ptr<QuantLib::BlackKarasinski> modelBK(new QuantLib::BlackKarasinski(rhTermStructure));
+        QuantLib::ext::shared_ptr<QuantLib::BlackKarasinski> modelBK(new QuantLib::BlackKarasinski(rhTermStructure));
         Rprintf((char*)"Black-Karasinski (tree) calibration\n");
         for (i=0; i<swaptions.size(); i++)
-            swaptions[i]->setPricingEngine(boost::shared_ptr<QuantLib::PricingEngine>(new QuantLib::TreeSwaptionEngine(modelBK,grid)));
+            swaptions[i]->setPricingEngine(QuantLib::ext::shared_ptr<QuantLib::PricingEngine>(new QuantLib::TreeSwaptionEngine(modelBK,grid)));
         calibrateModel(modelBK, swaptions, 0.05, swaptionMat, swapLengths, swaptionVols);
 
-        boost::shared_ptr<QuantLib::PricingEngine> engine(new QuantLib::TreeSwaptionEngine(modelBK, 50));
+        QuantLib::ext::shared_ptr<QuantLib::PricingEngine> engine(new QuantLib::TreeSwaptionEngine(modelBK, 50));
         QuantLib::Swaption bermudanSwaption(mySwap, bermudaExercise);
         bermudanSwaption.setPricingEngine(engine);
         return Rcpp::List::create(Rcpp::Named("a") = modelBK->params()[0],
